/**
 * Copyright (c) 2005-2012 EBMWebsourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.IOException;
import java.net.URL;
import java.util.logging.Logger;

import org.junit.Test;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.jbidescriptor.CDKJBIDescriptorBuilder;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests about {@link AbstractComponent}.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AbstractComponentTest {

    private final String DEFAULT_STRING_VALUE = "default-value";

    private final boolean DEFAULT_BOOL_VALUE = Boolean.TRUE.booleanValue();

    private final int DEFAULT_INT_VALUE = 9562;

    private final long DEFAULT_LONG_VALUE = 5468495397624564l;

    private final String STRING_VALUE = "abcd";

    private final String PARAM_NAME_NULL_VALUE = "param-null-that-is-not-provided";

    private final String PARAM_NAME_EMPTY_VALUE = "empty-value";

    private final String PARAM_NAME_SPACE_VALUE = "space-value";

    private final String PARAM_NAME_STRING_VALUE_SUFFIXED_BY_SPACE = "abcd-space";

    private final String PARAM_NAME_STRING_VALUE_PREFIXED_BY_SPACE = "space-abcd";

    private final String PARAM_NAME_TRIMMED_STRING_VALUE = "space-abcd";

    private static final Logger LOGGER = Logger.getLogger(AbstractComponentTest.class.getName());

    /**
     * Check processing of:
     * <ul>
     * <li>method {@link AbstractComponent#getParameterAsNotEmptyTrimmedString(String, String)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; 'abcd',</li>
     * <li>value ' abcd' --&gt; 'abcd',</li>
     * <li>value 'abcd ' --&gt; 'abcd'.</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsTrimmedString(String, String)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; '',</li>
     * <li>value ' ' --&gt; '',</li>
     * <li>value 'abcd' --&gt; 'abcd',</li>
     * <li>value ' abcd' --&gt; 'abcd',</li>
     * <li>value 'abcd ' --&gt; 'abcd'.</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsBoolean(String, boolean)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value 'true' --&gt; true,</li>
     * <li>value 'true ' --&gt; true,</li>
     * <li>value 'TRUE' --&gt; true,</li>
     * <li>value 'tRUe' --&gt; true,</li>
     * <li>value 'false' --&gt; true,</li>
     * <li>value ' false' --&gt; true,</li>
     * <li>value 'FALSE' --&gt; true,</li>
     * <li>value 'faLSe ' --&gt; false.</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsInteger(String, int)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value '1a2' --&gt; default value,</li>
     * <li>value '123' --&gt; 123</li>
     * <li>value '0' --&gt; 0</li>
     * <li>value '-789' --&gt; -789</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsPositiveInteger(String, int)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value '1a2' --&gt; default value,</li>
     * <li>value '123' --&gt; 123</li>
     * <li>value '0' --&gt; 0</li>
     * <li>value '-789' --&gt; default value</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsStrictPositiveInteger(String, int)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value '1a2' --&gt; default value,</li>
     * <li>value '123' --&gt; 123</li>
     * <li>value '0' --&gt; default value</li>
     * <li>value '-789' --&gt; default value</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsLong(String, long)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value '1a2' --&gt; default value,</li>
     * <li>value '123' --&gt; 123</li>
     * <li>value '0' --&gt; 0</li>
     * <li>value '-789' --&gt; -789</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsPositiveLong(String, long)} against:
     * <ul>
     * <li>value 'null' --&gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value '1a2' --&gt; default value,</li>
     * <li>value '123' --&gt; 123</li>
     * <li>value '0' --&gt; 0</li>
     * <li>value '-789' --&gt; default value</li>
     * </ul>
     * </li>
     * <li>method {@link AbstractComponent#getParameterAsStrictPositiveLong(String, long)} against:
     * <ul>
     * <li>value 'null' --gt; default value,</li>
     * <li>empty --&gt; default value,</li>
     * <li>value ' ' --&gt; default value,</li>
     * <li>value 'abcd' --&gt; default value,</li>
     * <li>value '1a2' --&gt; default value,</li>
     * <li>value '123' --&gt; 123</li>
     * <li>value '0' --&gt; default value</li>
     * <li>value '-789' --&gt; default value</li>
     * </ul>
     * </li>
     * </ul>
     */
    @Test
    public void componentParameterValues() throws JBIDescriptorException, IOException {

        final URL jbiXmlUrl = Thread.currentThread().getContextClassLoader()
                .getResource("descriptors/jbiComponent.xml");
        assertNotNull("JBI descriptor 'component-parameter-values.xml' containing parameters is not found", jbiXmlUrl);

        final Jbi jbi = CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(jbiXmlUrl.openStream());
        final ConfigurationExtensions cfgExt = new ConfigurationExtensions(jbi.getComponent().getAny());

        final AbstractComponent component = new AbstractComponent() {

            @Override
            protected AbstractServiceUnitManager createServiceUnitManager() {
                return null;
            }
        };
        ReflectionHelper.setFieldValue(AbstractComponent.class, component, "logger", LOGGER, false);
        ReflectionHelper.setFieldValue(AbstractComponent.class, component, "componentExtensions", cfgExt, false);

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsNotEmptyTrimmedString'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_STRING_VALUE,
                component.getParameterAsNotEmptyTrimmedString(PARAM_NAME_NULL_VALUE, DEFAULT_STRING_VALUE));
        assertEquals(DEFAULT_STRING_VALUE,
                component.getParameterAsNotEmptyTrimmedString(PARAM_NAME_EMPTY_VALUE, DEFAULT_STRING_VALUE));
        assertEquals(DEFAULT_STRING_VALUE,
                component.getParameterAsNotEmptyTrimmedString(PARAM_NAME_SPACE_VALUE, DEFAULT_STRING_VALUE));
        assertEquals(STRING_VALUE, component
                .getParameterAsNotEmptyTrimmedString(PARAM_NAME_STRING_VALUE_SUFFIXED_BY_SPACE, DEFAULT_STRING_VALUE));
        assertEquals(STRING_VALUE, component
                .getParameterAsNotEmptyTrimmedString(PARAM_NAME_STRING_VALUE_PREFIXED_BY_SPACE, DEFAULT_STRING_VALUE));
        assertEquals(STRING_VALUE,
                component.getParameterAsNotEmptyTrimmedString(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_STRING_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsTrimmedString'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_STRING_VALUE,
                component.getParameterAsTrimmedString(PARAM_NAME_NULL_VALUE, DEFAULT_STRING_VALUE));
        assertEquals("", component.getParameterAsTrimmedString(PARAM_NAME_EMPTY_VALUE, DEFAULT_STRING_VALUE));
        assertEquals("", component.getParameterAsTrimmedString(PARAM_NAME_SPACE_VALUE, DEFAULT_STRING_VALUE));
        assertEquals(STRING_VALUE,
                component.getParameterAsTrimmedString(PARAM_NAME_STRING_VALUE_SUFFIXED_BY_SPACE, DEFAULT_STRING_VALUE));
        assertEquals(STRING_VALUE,
                component.getParameterAsTrimmedString(PARAM_NAME_STRING_VALUE_PREFIXED_BY_SPACE, DEFAULT_STRING_VALUE));
        assertEquals(STRING_VALUE,
                component.getParameterAsNotEmptyTrimmedString(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_STRING_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsBoolean'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_BOOL_VALUE, component.getParameterAsBoolean(PARAM_NAME_NULL_VALUE, DEFAULT_BOOL_VALUE));
        assertEquals(DEFAULT_BOOL_VALUE, component.getParameterAsBoolean(PARAM_NAME_EMPTY_VALUE, DEFAULT_BOOL_VALUE));
        assertEquals(DEFAULT_BOOL_VALUE, component.getParameterAsBoolean(PARAM_NAME_SPACE_VALUE, DEFAULT_BOOL_VALUE));
        assertEquals(DEFAULT_BOOL_VALUE,
                component.getParameterAsBoolean(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.TRUE.booleanValue(),
                component.getParameterAsBoolean("bool-true-upper-case", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.FALSE.booleanValue(),
                component.getParameterAsBoolean("bool-false-upper-case", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.TRUE.booleanValue(),
                component.getParameterAsBoolean("bool-true-lower-case", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.TRUE.booleanValue(),
                component.getParameterAsBoolean("bool-true-lower-case-suffixed-by-space", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.FALSE.booleanValue(),
                component.getParameterAsBoolean("bool-false-lower-case", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.FALSE.booleanValue(),
                component.getParameterAsBoolean("bool-false-lower-case-prefixed-by-space", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.TRUE.booleanValue(),
                component.getParameterAsBoolean("bool-true-mixed-case", DEFAULT_BOOL_VALUE));
        assertEquals(Boolean.FALSE.booleanValue(),
                component.getParameterAsBoolean("bool-false-mixed-case", DEFAULT_BOOL_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsInteger'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsInteger(PARAM_NAME_NULL_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsInteger(PARAM_NAME_EMPTY_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsInteger(PARAM_NAME_SPACE_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsInteger(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsInteger("alpha-numeric", DEFAULT_INT_VALUE));
        assertEquals(123, component.getParameterAsInteger("positive-value", DEFAULT_INT_VALUE));
        assertEquals(0, component.getParameterAsInteger("value-0", DEFAULT_INT_VALUE));
        assertEquals(-789, component.getParameterAsInteger("negative-value", DEFAULT_INT_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsPositiveInteger'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsPositiveInteger(PARAM_NAME_NULL_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsPositiveInteger(PARAM_NAME_EMPTY_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsPositiveInteger(PARAM_NAME_SPACE_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsPositiveInteger(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsPositiveInteger("alpha-numeric", DEFAULT_INT_VALUE));
        assertEquals(123, component.getParameterAsPositiveInteger("positive-value", DEFAULT_INT_VALUE));
        assertEquals(0, component.getParameterAsPositiveInteger("value-0", DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsPositiveInteger("negative-value", DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsPositiveInteger("long-value", DEFAULT_INT_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsStrictPositiveInteger'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsStrictPositiveInteger(PARAM_NAME_NULL_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsStrictPositiveInteger(PARAM_NAME_EMPTY_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsStrictPositiveInteger(PARAM_NAME_SPACE_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsStrictPositiveInteger(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsStrictPositiveInteger("alpha-numeric", DEFAULT_INT_VALUE));
        assertEquals(123, component.getParameterAsStrictPositiveInteger("positive-value", DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE, component.getParameterAsStrictPositiveInteger("value-0", DEFAULT_INT_VALUE));
        assertEquals(DEFAULT_INT_VALUE,
                component.getParameterAsStrictPositiveInteger("negative-value", DEFAULT_INT_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsLong'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsLong(PARAM_NAME_NULL_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsLong(PARAM_NAME_EMPTY_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsLong(PARAM_NAME_SPACE_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsLong(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsLong("alpha-numeric", DEFAULT_LONG_VALUE));
        assertEquals(123, component.getParameterAsLong("positive-value", DEFAULT_LONG_VALUE));
        assertEquals(0, component.getParameterAsLong("value-0", DEFAULT_LONG_VALUE));
        assertEquals(-789, component.getParameterAsLong("negative-value", DEFAULT_LONG_VALUE));
        assertEquals(1235495397624564456l,
                component.getParameterAsPositiveLong("long-value", DEFAULT_LONG_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsPositiveLong'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsPositiveLong(PARAM_NAME_NULL_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsPositiveLong(PARAM_NAME_EMPTY_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsPositiveLong(PARAM_NAME_SPACE_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsPositiveLong(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsPositiveLong("alpha-numeric", DEFAULT_LONG_VALUE));
        assertEquals(123, component.getParameterAsPositiveLong("positive-value", DEFAULT_LONG_VALUE));
        assertEquals(0, component.getParameterAsPositiveLong("value-0", DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsPositiveLong("negative-value", DEFAULT_LONG_VALUE));

        // ----------------------------------------------------------------
        // Assertions about method 'getParameterAsStrictPositiveLong'
        // ----------------------------------------------------------------
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsStrictPositiveLong(PARAM_NAME_NULL_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsStrictPositiveLong(PARAM_NAME_EMPTY_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsStrictPositiveLong(PARAM_NAME_SPACE_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsStrictPositiveLong(PARAM_NAME_TRIMMED_STRING_VALUE, DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsStrictPositiveLong("alpha-numeric", DEFAULT_LONG_VALUE));
        assertEquals(123, component.getParameterAsStrictPositiveLong("positive-value", DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE, component.getParameterAsStrictPositiveLong("value-0", DEFAULT_LONG_VALUE));
        assertEquals(DEFAULT_LONG_VALUE,
                component.getParameterAsStrictPositiveLong("negative-value", DEFAULT_LONG_VALUE));
    }

}
