/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.listener;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.DeliveryChannel;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;

import org.easymock.EasyMock;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.jbidescriptor.CDKJBIDescriptorBuilder;
import org.ow2.petals.component.framework.jbidescriptor.generated.ClassPath;
import org.ow2.petals.component.framework.jbidescriptor.generated.Component;
import org.ow2.petals.component.framework.jbidescriptor.generated.ComponentClassName;
import org.ow2.petals.component.framework.jbidescriptor.generated.ComponentInterceptors;
import org.ow2.petals.component.framework.jbidescriptor.generated.ComponentType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Identification;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.jbidescriptor.generated.Runtimepositivestrictint;
import org.ow2.petals.component.framework.jbidescriptor.generated.Runtimepositivestrictlong;
import org.ow2.petals.component.framework.jbidescriptor.generated.SettableanyURI;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settablepositivestrictint;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settablepositivestrictlong;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;

/**
 * Unit tests of {@link AbstractListener}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AbstractListenerTest {

    private static final Logger LOG = Logger.getLogger(AbstractListenerTest.class.getName());

    private static final long HARDCODED_TIMEOUT_VALUE = 123456;

    private static final String PLACEHOLDER_TIMEOUT = "placeholder.timeout";

    private static final long PLACEHOLDER_TIMEOUT_VALUE = 987654321;

    private static final long PLACEHOLDER_TIMEOUT_NEW_VALUE = 654787932;

    @Rule
    public final TemporaryFolder tmpFolder = new TemporaryFolder();

    private File propertiesFile;

    @Before
    public void createPropertiesFile() throws IOException {
        this.propertiesFile = this.tmpFolder.newFile();
        this.createPropertiesFile(PLACEHOLDER_TIMEOUT_VALUE);
    }

    private void createPropertiesFile(final long placeHolderTimeoutValue) throws FileNotFoundException, IOException {

        final Properties placeholders = new Properties();
        placeholders.setProperty(PLACEHOLDER_TIMEOUT, String.valueOf(placeHolderTimeoutValue));
        try (final FileOutputStream fos = new FileOutputStream(this.propertiesFile)) {
            placeholders.store(fos, "");
        }
    }

    private class AbstractComponentForTest extends AbstractComponent {

        public AbstractComponentForTest()
                throws MissingResourceException, JBIException, JBIDescriptorException, IOException,
                InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {

            final Jbi jbiDescriptor = new Jbi();
            jbiDescriptor.setVersion(new BigDecimal(1));
            final Component jbiComponent = new Component();
            jbiDescriptor.setComponent(jbiComponent);
            jbiComponent.setType(ComponentType.BINDING_COMPONENT);
            final Identification jbiIdentification = new Identification();
            jbiIdentification.setName("comp-name");
            jbiIdentification.setDescription("comp description");
            jbiComponent.setIdentification(jbiIdentification);
            final ComponentClassName compClassName = new ComponentClassName();
            compClassName.setContent("comp.class.name");
            jbiComponent.setComponentClassName(compClassName);
            final ClassPath compClassPath = new ClassPath();
            jbiComponent.setComponentClassPath(compClassPath);
            compClassPath.getPathElement().add("comp-path-elt");
            jbiComponent.setBootstrapClassName("com.bootstrap.class.name");
            jbiComponent.setBootstrapClassPath(compClassPath);
            final Runtimepositivestrictint acceptorPoolSize = new Runtimepositivestrictint();
            acceptorPoolSize.setValue(5);
            jbiComponent.setAcceptorPoolSize(acceptorPoolSize);
            final Settablepositivestrictint acceptorRetryNumber = new Settablepositivestrictint();
            acceptorRetryNumber.setValue(3);
            jbiComponent.setAcceptorRetryNumber(acceptorRetryNumber);
            final Settablepositivestrictlong acceptorRetryWait = new Settablepositivestrictlong();
            acceptorRetryWait.setValue(1000);
            jbiComponent.setAcceptorRetryWait(acceptorRetryWait);
            final Runtimepositivestrictlong acceptorStopMaxWait = new Runtimepositivestrictlong();
            acceptorStopMaxWait.setValue(1000);
            jbiComponent.setAcceptorStopMaxWait(acceptorStopMaxWait);
            final Runtimepositivestrictint processorPoolSize = new Runtimepositivestrictint();
            processorPoolSize.setValue(10);
            jbiComponent.setProcessorPoolSize(processorPoolSize);
            final Runtimepositivestrictint processorMaxPoolSize = new Runtimepositivestrictint();
            processorMaxPoolSize.setValue(10);
            jbiComponent.setProcessorMaxPoolSize(processorMaxPoolSize);
            final Runtimepositivestrictint processorKeepAliveTime = new Runtimepositivestrictint();
            processorKeepAliveTime.setValue(1000);
            jbiComponent.setProcessorKeepAliveTime(processorKeepAliveTime);
            final Runtimepositivestrictlong processorStopMaxWait = new Runtimepositivestrictlong();
            processorStopMaxWait.setValue(1000);
            jbiComponent.setProcessorStopMaxWait(processorStopMaxWait);
            final Settablepositivestrictlong timeBetweenAsyncCleanerRuns = new Settablepositivestrictlong();
            timeBetweenAsyncCleanerRuns.setValue(1000);
            jbiComponent.setTimeBeetweenAsyncCleanerRuns(timeBetweenAsyncCleanerRuns);
            final Settablepositivestrictlong monitoringSamplingPeriod = new Settablepositivestrictlong();
            monitoringSamplingPeriod.setValue(1000);
            jbiComponent.setMonitoringSamplingPeriod(monitoringSamplingPeriod);
            final ComponentInterceptors compInterceptors = new ComponentInterceptors();
            jbiComponent.setComponentInterceptors(compInterceptors);
            jbiComponent.setJbiListenerClassName("com.listener.class.name");
            final SettableanyURI propertiesFileURI = new SettableanyURI();
            propertiesFileURI.setValue(propertiesFile.toURI().toString());
            jbiComponent.setPropertiesFile(propertiesFileURI);

            final File metaInfDir = tmpFolder.newFolder("META-INF");
            final File jbiDescriptorFile = tmpFolder.newFile(JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE);
            try (final FileOutputStream fos = new FileOutputStream(jbiDescriptorFile)) {
                CDKJBIDescriptorBuilder.getInstance().writeXMLJBIdescriptor(jbiDescriptor, fos);
            }

            final ComponentContext compCtx = EasyMock.createMock(ComponentContext.class);
            EasyMock.expect(compCtx.getLogger("", null)).andReturn(LOG).anyTimes();
            final DeliveryChannel channel = EasyMock.createMock(DeliveryChannel.class);
            EasyMock.expect(channel.createExchangeFactory()).andReturn(null).anyTimes();
            EasyMock.expect(compCtx.getDeliveryChannel()).andReturn(channel).anyTimes();
            EasyMock.expect(compCtx.getInstallRoot()).andReturn(jbiDescriptorFile.getParentFile().getParent()).anyTimes();
            EasyMock.expect(compCtx.getComponentName()).andReturn("comp-name").anyTimes();
            EasyMock.expect(compCtx.getWorkspaceRoot()).andReturn(tmpFolder.newFolder().getAbsolutePath()).anyTimes();
            final MBeanNames mbeanNames = EasyMock.createMock(MBeanNames.class);
            EasyMock.expect(mbeanNames.createCustomComponentMBeanName("monitoring_comp-name")).andReturn(null)
                    .anyTimes();
            EasyMock.expect(compCtx.getMBeanNames()).andReturn(mbeanNames).anyTimes();
            final MBeanServer mbeanServer = EasyMock.createMock(MBeanServer.class);
            EasyMock.expect(mbeanServer.isRegistered(null)).andReturn(false);
            EasyMock.expect(mbeanServer.registerMBean(EasyMock.anyObject(), (ObjectName) EasyMock.anyObject()))
                    .andReturn(null).anyTimes();
            EasyMock.expect(compCtx.getMBeanServer()).andReturn(mbeanServer).anyTimes();
            EasyMock.replay(compCtx, channel, mbeanNames, mbeanServer);
            this.init(compCtx);
        }

        @Override
        protected AbstractServiceUnitManager createServiceUnitManager() {
            return null;
        }

    }

    private class AbstractListenerForTest extends AbstractListener {
        public AbstractListenerForTest()
                throws MissingResourceException, JBIException, JBIDescriptorException, IOException,
                InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
            this.init(new AbstractComponentForTest());
        }

    }

    /**
     * <p>
     * Check method {@link AbstractListener#getTimeout(Object)} against {@link Consumes}.
     * </p>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>if no timeout is defined: default timeout value is returned,</li>
     * <li>if a hard-coded value is set: this value is returned,</li>
     * <li>if a NaN value is set: default timeout is returned,</li>
     * <li>if a placeholder value is set: the placeholder value is returned,</li>
     * <li>if the placeholder value is changed: the placeholder new value is returned.</li>
     * </ul>
     */
    @Test
    public void getTimeout_Consumes()
            throws MissingResourceException, JBIException, JBIDescriptorException, IOException,
            InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {

        final AbstractListener listener = new AbstractListenerForTest();

        // No timeout define in consumer
        assertEquals(Constants.Component.DEFAULT_SEND_TIMEOUT, listener.getTimeout(new Consumes()));

        // Timeout hardcoded in consumer
        final Consumes hardCodedConsumer = new Consumes();
        hardCodedConsumer.setTimeout(String.valueOf(HARDCODED_TIMEOUT_VALUE));
        assertEquals(HARDCODED_TIMEOUT_VALUE, listener.getTimeout(hardCodedConsumer));

        // Timeout NaN in provider
        final Consumes NaNCodedConsumer = new Consumes();
        NaNCodedConsumer.setTimeout("NaN");
        assertEquals(Constants.Component.DEFAULT_SEND_TIMEOUT, listener.getTimeout(NaNCodedConsumer));

        // Timeout with placeholder in consumer
        final Consumes placeholderConsumer = new Consumes();
        placeholderConsumer.setTimeout(String.format("${%s}", PLACEHOLDER_TIMEOUT));
        assertEquals(PLACEHOLDER_TIMEOUT_VALUE, listener.getTimeout(placeholderConsumer));
        this.createPropertiesFile(PLACEHOLDER_TIMEOUT_NEW_VALUE);
        listener.getComponent().reloadPlaceHolders();
        assertEquals(PLACEHOLDER_TIMEOUT_NEW_VALUE, listener.getTimeout(placeholderConsumer));
    }

    /**
     * <p>
     * Check method {@link AbstractListener#getTimeout(Object)} against {@link Provides}.
     * </p>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>if no timeout is defined: default timeout value is returned,</li>
     * <li>if a hard-coded value is set: this value is returned,</li>
     * <li>if a NaN value is set: default timeout is returned,</li>
     * <li>if a placeholder value is set: the placeholder value is returned,</li>
     * <li>if the placeholder value is changed: the placeholder new value is returned.</li>
     * </ul>
     */
    @Test
    public void getTimeout_Provides() throws MissingResourceException, JBIException, JBIDescriptorException,
            IOException, InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {

        final AbstractListener listener = new AbstractListenerForTest();

        // No timeout define in provider
        assertEquals(Constants.Component.DEFAULT_SEND_TIMEOUT, listener.getTimeout(new Provides()));

        // Timeout hardcoded in provider
        final Provides hardCodedProvider = new Provides();
        hardCodedProvider.setTimeout(String.valueOf(HARDCODED_TIMEOUT_VALUE));
        assertEquals(HARDCODED_TIMEOUT_VALUE, listener.getTimeout(hardCodedProvider));

        // Timeout NaN in provider
        final Provides NaNCodedProvider = new Provides();
        NaNCodedProvider.setTimeout("NaN");
        assertEquals(Constants.Component.DEFAULT_SEND_TIMEOUT, listener.getTimeout(NaNCodedProvider));

        // Timeout with placeholder in provider
        final Provides placeholderProvider = new Provides();
        placeholderProvider.setTimeout(String.format("${%s}", PLACEHOLDER_TIMEOUT));
        assertEquals(PLACEHOLDER_TIMEOUT_VALUE, listener.getTimeout(placeholderProvider));
        this.createPropertiesFile(PLACEHOLDER_TIMEOUT_NEW_VALUE);
        listener.getComponent().reloadPlaceHolders();
        assertEquals(PLACEHOLDER_TIMEOUT_NEW_VALUE, listener.getTimeout(placeholderProvider));
    }

    /**
     * <p>
     * Check method {@link AbstractListener#getTimeout(Object)} against a service type unpexected (neither
     * {@link Consumes} nor {@link Provides}).
     * </p>
     * <p>
     * Expected result: Default timeout value is returned.
     * </p>
     */
    @Test
    public void getTimeout_Other() throws MissingResourceException, JBIException, JBIDescriptorException, IOException,
            InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {

        final AbstractListener listener = new AbstractListenerForTest();

        // Unexpected service type
        assertEquals(Constants.Component.DEFAULT_SEND_TIMEOUT, listener.getTimeout(new Object()));
    }

    /**
     * <p>
     * Check method {@link AbstractListener#getTimeout(Object)} against no service provided.
     * </p>
     * <p>
     * Expected result: Default timeout value is returned.
     * </p>
     */
    @Test
    public void getTimeout_Null() throws MissingResourceException, JBIException, JBIDescriptorException, IOException,
            InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {

        final AbstractListener listener = new AbstractListenerForTest();

        // No timeout define in consumer
        assertEquals(Constants.Component.DEFAULT_SEND_TIMEOUT, listener.getTimeout(null));
    }

}
