/**
 * Copyright (c) 2010-2012 EBMWebsourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class XMLUtilTest {

    private static final String QUALIFICATION_NS = "http://petals.ow2.org/bc/ged";

    private static final String QUALIFICATION_LOCALPART = "archiver";

    @Test
    public void findVariableTrimmed() throws ParserConfigurationException, SAXException, IOException {

        final String baseMsg = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<getResponse>"
                + "<!-- comment after getResponse before base node -->" + "<test>linux rocks</test>"
                + "<base><!-- yet xml comment -->${content}</base>" + "</getResponse>";

        final Node nodeFound = XMLUtilTest.findVariable(baseMsg);

        assertEquals("Check node parent", "base", nodeFound.getParentNode().getLocalName());
    }

    @Test
    public void findVariableNotTrimmed() throws ParserConfigurationException, SAXException, IOException {

        final StringBuilder baseMsg = new StringBuilder();
        baseMsg.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        baseMsg.append('\n');
        baseMsg.append("<getResponse>");
        baseMsg.append('\n');
        baseMsg.append("<!-- comment after getResponse before base node -->");
        baseMsg.append('\n');
        baseMsg.append("<test>linux rocks</test>");
        baseMsg.append('\n');
        baseMsg.append("<base>");
        baseMsg.append('\n');
        baseMsg.append("<!-- yet xml comment -->");
        baseMsg.append('\n');
        baseMsg.append("${content}");
        baseMsg.append('\n');
        baseMsg.append("</base>");
        baseMsg.append('\n');
        baseMsg.append("</getResponse>");

        final Node nodeFound = XMLUtilTest.findVariable(baseMsg.toString());

        assertEquals("Check node parent", "base", nodeFound.getParentNode().getLocalName());
    }

    private static Node findVariable(final String baseMsg)
            throws ParserConfigurationException, SAXException, IOException {

        final Document resultDoc = XMLUtilTest.buildDocument(baseMsg);

            assertNull("Check node not found", XMLUtil.findVariable(resultDoc.getChildNodes(), "${attachments}"));
            final Node nodeFound = XMLUtil.findVariable(resultDoc.getChildNodes(), "${content}");

            assertNotNull("Check node found", nodeFound);
            assertEquals("Check node type", Node.TEXT_NODE, nodeFound.getNodeType());

            return nodeFound;
    }

    private static Document buildDocument(final String documentStr) throws SAXException, IOException {
        final DocumentBuilder builder = DocumentBuilders.takeDocumentBuilder();
        try {
            return builder.parse(new ByteArrayInputStream(documentStr.getBytes()));
        } finally {
            DocumentBuilders.releaseDocumentBuilder(builder);
        }
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} with a normal value.
     */
    @Test
    public void getElementValueAsQName_nominal() throws SAXException, IOException {
        final StringBuilder documentStr = new StringBuilder();
        documentStr.append("<service-provider-operation xmlns:tech=\"").append(QUALIFICATION_NS).append("\">tech:")
                .append(QUALIFICATION_LOCALPART).append("</service-provider-operation>");
        final Document doc = XMLUtilTest.buildDocument(documentStr.toString());
        final QName qualifiedValue = XMLUtil.getElementValueAsQName(doc.getDocumentElement());
        assertEquals(new QName(QUALIFICATION_NS, QUALIFICATION_LOCALPART), qualifiedValue);
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} when the prefix is unknown
     */
    @Test
    public void getElementValueAsQName_unknownPrefix() throws SAXException, IOException {
        final StringBuilder documentStr = new StringBuilder();
        documentStr.append("<service-provider-operation xmlns:tech=\"").append(QUALIFICATION_NS)
                .append("\">unknown-prefix:").append(QUALIFICATION_LOCALPART).append("</service-provider-operation>");
        final Document doc = XMLUtilTest.buildDocument(documentStr.toString());
        final QName qualifiedValue = XMLUtil.getElementValueAsQName(doc.getDocumentElement());
        assertEquals(new QName("", QUALIFICATION_LOCALPART), qualifiedValue);
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} when the value starts with ':' (the prefix part is
     * missing)
     */
    @Test
    public void getElementValueAsQName_prefixMising() throws SAXException, IOException {
        final StringBuilder documentStr = new StringBuilder();
        documentStr.append("<service-provider-operation xmlns:tech=\"").append(QUALIFICATION_NS).append("\">:")
                .append(QUALIFICATION_LOCALPART).append("</service-provider-operation>");
        final Document doc = XMLUtilTest.buildDocument(documentStr.toString());
        final QName qualifiedValue = XMLUtil.getElementValueAsQName(doc.getDocumentElement());
        assertEquals(new QName("", QUALIFICATION_LOCALPART), qualifiedValue);
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} when no prefix is set.
     */
    @Test
    public void getElementValueAsQName_noPrefixSet() throws SAXException, IOException {
        final StringBuilder documentStr = new StringBuilder();
        documentStr.append("<service-provider-operation>").append(QUALIFICATION_LOCALPART)
                .append("</service-provider-operation>");
        final Document doc = XMLUtilTest.buildDocument(documentStr.toString());
        final QName qualifiedValue = XMLUtil.getElementValueAsQName(doc.getDocumentElement());
        assertEquals(new QName("", QUALIFICATION_LOCALPART), qualifiedValue);
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} when the node is {@code null}.
     */
    @Test
    public void getElementValueAsQName_nullNode() throws SAXException, IOException {
        assertNull(XMLUtil.getElementValueAsQName(null));
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} when the XML tag of the node is closed without value.
     */
    @Test
    public void getElementValueAsQName_tagClosed() throws SAXException, IOException {
        final StringBuilder documentStr = new StringBuilder();
        documentStr.append("<service-provider-operation />");
        final Document doc = XMLUtilTest.buildDocument(documentStr.toString());
        assertNull(XMLUtil.getElementValueAsQName(doc.getDocumentElement()));
    }

    /**
     * Check method {@link XMLUtil#getElementValueAsQName(Node)} when the XML tag value of the node is empty.
     */
    @Test
    public void getElementValueAsQName_emptyValue() throws SAXException, IOException {
        final StringBuilder documentStr = new StringBuilder();
        documentStr.append("<service-provider-operation></service-provider-operation>");
        final Document doc = XMLUtilTest.buildDocument(documentStr.toString());
        assertNull(XMLUtil.getElementValueAsQName(doc.getDocumentElement()));
    }
}
